@@ -27,27 +27,10 @@ char32_t notcurses_getc_nblock(struct notcurses* n, ncinput* ni);
 char32_t notcurses_getc_blocking(struct notcurses* n, ncinput* ni);
 int notcurses_inputready_fd(struct notcurses* n);
 typedef struct cell {
-  // These 32 bits are either a single-byte, single-character grapheme cluster
-  // (values 0--0x7f), or an offset into a per-ncplane attached pool of
-  // varying-length UTF-8 grapheme clusters. This pool may thus be up to 32MB.
-  uint32_t gcluster;          // 1 * 4b -> 4b
-  // CELL_STYLE_* attributes (16 bits) + 16 reserved bits
-  uint32_t attrword;          // + 4b -> 8b
-  // (channels & 0x8000000000000000ull): left half of wide character
-  // (channels & 0x4000000000000000ull): foreground is *not* "default color"
-  // (channels & 0x3000000000000000ull): foreground alpha (2 bits)
-  // (channels & 0x0f00000000000000ull): reserved, must be 0
-  // (channels & 0x00ffffff00000000ull): foreground in 3x8 RGB (rrggbb)
-  // (channels & 0x0000000080000000ull): right half of wide character
-  // (channels & 0x0000000040000000ull): background is *not* "default color"
-  // (channels & 0x0000000030000000ull): background alpha (2 bits)
-  // (channels & 0x000000000f000000ull): reserved, must be 0
-  // (channels & 0x0000000000ffffffull): background in 3x8 RGB (rrggbb)
-  // At render time, these 24-bit values are quantized down to terminal
-  // capabilities, if necessary. There's a clear path to 10-bit support should
-  // we one day need it, but keep things cagey for now. "default color" is
-  // best explained by color(3NCURSES). ours is the same concept. until the
-  // "not default color" bit is set, any color you load will be ignored.
+  uint32_t gcluster;          // 4B → 4B
+  uint8_t gcluster_backstop;  // 1B → 5B (8 bits of zero)
+  uint8_t reserved;           // 1B → 6B (8 reserved bits, ought be zero)
+  uint16_t stylemask;         // 2B → 8B (16 bits of NCSTYLE_* attributes)
   uint64_t channels;          // + 8b == 16b
 } cell;
 typedef enum {
@@ -62,24 +45,10 @@ typedef enum {
   NCLOGLEVEL_TRACE,   // there's probably a better way to do what you want
 } ncloglevel_e;
 typedef struct notcurses_options {
-  // The name of the terminfo database entry describing this terminal. If NULL,
-  // the environment variable TERM is used. Failure to open the terminal
-  // definition will result in failure to initialize notcurses.
   const char* termtype;
-  // If non-NULL, notcurses_render() will write each rendered frame to this
-  // FILE* in addition to outfp. This is used primarily for debugging.
   FILE* renderfp;
-  // Progressively higher log levels result in more logging to stderr. By
-  // default, nothing is printed to stderr once fullscreen service begins.
   ncloglevel_e loglevel;
-  // Desirable margins. If all are 0 (default), we will render to the entirety
-  // of the screen. If the screen is too small, we do what we can--this is
-  // strictly best-effort. Absolute coordinates are relative to the rendering
-  // area ((0, 0) is always the origin of the rendering area).
   int margin_t, margin_r, margin_b, margin_l;
-  // General flags; see NCOPTION_*. This is expressed as a bitfield so that
-  // future options can be added without reshaping the struct. Undefined bits
-  // must be set to 0.
   uint64_t flags;
 } notcurses_options;
 struct notcurses* notcurses_init(const notcurses_options*, FILE*);
@@ -90,23 +59,28 @@ int notcurses_render(struct notcurses*);
 int notcurses_render_to_file(struct notcurses* nc, FILE* fp);
 struct ncplane* notcurses_stdplane(struct notcurses*);
 const struct ncplane* notcurses_stdplane_const(const struct notcurses* nc);
+void ncplane_set_channels(struct ncplane* nc, uint64_t channels);
+void ncplane_set_attr(struct ncplane* n, unsigned stylebits);
 int ncplane_set_base_cell(struct ncplane* ncp, const cell* c);
 int ncplane_set_base(struct ncplane* ncp, const char* egc, uint32_t attrword, uint64_t channels);
 int ncplane_base(struct ncplane* ncp, cell* c);
 struct ncplane* notcurses_top(struct notcurses* n);
 void notcurses_drop_planes(struct notcurses* nc);
 int notcurses_refresh(struct notcurses* n, int* restrict y, int* restrict x);
-struct ncplane* ncplane_new(struct notcurses* nc, int rows, int cols, int yoff, int xoff, void* opaque);
-struct ncplane* ncplane_bound(struct ncplane* n, int rows, int cols, int yoff, int xoff, void* opaque);
 struct ncplane* ncplane_reparent(struct ncplane* n, struct ncplane* newparent);
 typedef enum {
   NCALIGN_LEFT,
   NCALIGN_CENTER,
   NCALIGN_RIGHT,
 } ncalign_e;
+struct ncplane* ncplane_new(struct notcurses* nc, int rows, int cols, int yoff, int xoff, void* opaque);
+struct ncplane* ncplane_new_named(struct notcurses* nc, int rows, int cols, int yoff, int xoff, void* opaque, const char* name);
+struct ncplane* ncplane_bound(struct ncplane* n, int rows, int cols, int yoff, int xoff, void* opaque);
+struct ncplane* ncplane_bound_named(struct ncplane* n, int rows, int cols, int yoff, int xoff, void* opaque, const char* name);
 struct ncplane* ncplane_aligned(struct ncplane* n, int rows, int cols, int yoff, ncalign_e align, void* opaque);
+struct ncplane* ncplane_aligned_named(struct ncplane* n, int rows, int cols, int yoff, ncalign_e align, void* opaque, const char* name);
 unsigned notcurses_supported_styles(const struct notcurses* nc);
-int notcurses_palette_size(const struct notcurses* nc);
+unsigned notcurses_palette_size(const struct notcurses* nc);
 bool notcurses_cantruecolor(const struct notcurses* nc);
 bool notcurses_canfade(const struct notcurses* nc);
 bool notcurses_canchangecolor(const struct notcurses* nc);
@@ -130,10 +104,10 @@ void ncplane_move_bottom(struct ncplane* n);
 int ncplane_move_below(struct ncplane* restrict n, struct ncplane* restrict below);
 int ncplane_move_above(struct ncplane* restrict n, struct ncplane* restrict above);
 struct ncplane* ncplane_below(struct ncplane* n);
-char* notcurses_at_yx(struct notcurses* nc, int yoff, int xoff, uint32_t* attrword, uint64_t* channels);
-char* ncplane_at_cursor(struct ncplane* n, uint32_t* attrword, uint64_t* channels);
+char* notcurses_at_yx(struct notcurses* nc, int yoff, int xoff, uint16_t* stylemask, uint64_t* channels);
+char* ncplane_at_cursor(struct ncplane* n, uint16_t* stylemask, uint64_t* channels);
 int ncplane_at_cursor_cell(struct ncplane* n, cell* c);
-char* ncplane_at_yx(const struct ncplane* n, int y, int x, uint32_t* attrword, uint64_t* channels);
+char* ncplane_at_yx(const struct ncplane* n, int y, int x, uint16_t* stylemask, uint64_t* channels);
 int ncplane_at_yx_cell(struct ncplane* n, int y, int x, cell* c);
 typedef enum {
   NCBLIT_1x1,     // full block                █
@@ -154,7 +128,7 @@ void* ncplane_userptr(struct ncplane* n);
 int ncplane_resize(struct ncplane* n, int keepy, int keepx, int keepleny,
                        int keeplenx, int yoff, int xoff, int ylen, int xlen);
 uint64_t ncplane_channels(struct ncplane* n);
-uint32_t ncplane_attr(struct ncplane* n);
+uint16_t ncplane_attr(struct ncplane* n);
 unsigned ncplane_bchannel(struct ncplane* nc);
 unsigned ncplane_fchannel(struct ncplane* nc);
 unsigned ncplane_fg(struct ncplane* nc);
@@ -178,7 +152,6 @@ int ncplane_set_bg_palindex(struct ncplane* n, int idx);
 void ncplane_styles_set(struct ncplane* n, unsigned stylebits);
 void ncplane_styles_on(struct ncplane* n, unsigned stylebits);
 void ncplane_styles_off(struct ncplane* n, unsigned stylebits);
-unsigned ncplane_styles(const struct ncplane* n);
 typedef struct ncstats {
   uint64_t renders;          // number of successful notcurses_render() runs
   uint64_t failed_renders;   // number of aborted renders, should be 0
@@ -229,9 +202,7 @@ int cell_set_bg_alpha(cell* c, unsigned alpha);
 bool cell_double_wide_p(const cell* c);
 bool cell_simple_p(const cell* c);
 const char* cell_extended_gcluster(const struct ncplane* n, const cell* c);
-bool cell_noforeground_p(const cell* c);
 int cell_load_simple(struct ncplane* n, cell* c, char ch);
-uint32_t cell_egc_idx(const cell* c);
 unsigned cell_bchannel(const cell* cl);
 unsigned cell_fchannel(const cell* cl);
 uint64_t cell_set_bchannel(cell* cl, uint32_t channel);
@@ -314,8 +285,7 @@ typedef struct ncselector_options {
   char* secondary; // secondary may be NULL
   char* footer; // footer may be NULL
   struct ncselector_item* items; // initial items and descriptions
-  unsigned itemcount; // number of initial items and descriptions
-  // default item (selected at start), must be < itemcount unless 'itemcount'
+  // default item (selected at start), must be < itemcount unless itemcount
   // is 0, in which case 'defidx' must also be 0
   unsigned defidx;
   // maximum number of options to display at once, 0 to use all available space
@@ -347,7 +317,6 @@ typedef struct ncmultiselector_options {
   char* secondary; // secondary may be NULL
   char* footer; // footer may be NULL
   struct ncmselector_item* items; // initial items, descriptions, and statuses
-  unsigned itemcount; // number of items and descriptions, can't be 0
   // maximum number of options to display at once, 0 to use all available space
   unsigned maxdisplay;
   // exhaustive styling options
@@ -362,7 +331,7 @@ struct ncmultiselector* ncmultiselector_create(struct ncplane* n, int y, int x,
 int ncmultiselector_selected(struct ncmultiselector* n, bool* selected, unsigned count);
 struct ncplane* ncmultiselector_plane(struct ncmultiselector* n);
 bool ncmultiselector_offer_input(struct ncmultiselector* n, const struct ncinput* nc);
-void ncmultiselector_destroy(struct ncmultiselector* n, char** item);
+void ncmultiselector_destroy(struct ncmultiselector* n);
 struct ncmenu_item {
   char* desc;           // utf-8 menu item, NULL for horizontal separator
   ncinput shortcut;     // shortcut, all should be distinct
@@ -402,7 +371,7 @@ typedef struct ncreel_options {
 } ncreel_options;
 struct ncreel* ncreel_create(struct ncplane* nc, const ncreel_options* popts);
 struct ncplane* ncreel_plane(struct ncreel* pr);
-typedef int (*tabletcb)(struct nctablet* t, int begx, int begy, int maxx, int maxy, bool cliptop);
+typedef int (*tabletcb)(struct nctablet* t, bool cliptop);
 struct nctablet* ncreel_add(struct ncreel* pr, struct nctablet* after, struct nctablet* before, tabletcb cb, void* opaque);
 int ncreel_tabletcount(const struct ncreel* pr);
 int ncreel_del(struct ncreel* pr, struct nctablet* t);
@@ -470,7 +439,7 @@ typedef struct ncreader_options {
   uint64_t echannels; // channels used for empty space
   uint32_t tattrword; // attributes used for input
   uint32_t eattrword; // attributes used for empty space
-  char* egc;          // egc used for empty space
+  const char* egc;    // egc used for empty space
   int physrows;
   int physcols;
   unsigned flags;     // bitfield over NCREADER_OPTION_*
@@ -488,7 +457,7 @@ int ncplane_qrcode(struct ncplane* n, ncblitter_e blitter, int* ymax, int* xmax,
 struct ncdirect* ncdirect_init(const char* termtype, FILE* fp);
 int ncdirect_bg_rgb(struct ncdirect* n, unsigned r, unsigned g, unsigned b);
 int ncdirect_fg_rgb(struct ncdirect* n, unsigned r, unsigned g, unsigned b);
-int ncdirect_palette_size(const struct ncdirect* nc);
+unsigned ncdirect_palette_size(const struct ncdirect* nc);
 int ncdirect_putstr(struct ncdirect* nc, uint64_t channels, const char* utf8);
 int ncdirect_fg(struct ncdirect* n, unsigned rgb);
 int ncdirect_bg(struct ncdirect* n, unsigned rgb);
@@ -514,6 +483,8 @@ int ncdirect_double_box(struct ncdirect* n, uint64_t ul, uint64_t ur, uint64_t l
 bool ncdirect_canopen_images(const struct ncdirect* n);
 bool ncdirect_canutf8(const struct ncdirect* n);
 nc_err_e ncdirect_render_image(struct ncdirect* n, const char* filename, ncalign_e align, ncblitter_e blitter, ncscale_e scale);
+struct ncplane* ncplane_parent(struct ncplane* n);
+const struct ncplane* ncplane_parent_const(const struct ncplane* n);
 """)
 
 if __name__ == "__main__":
